#!/usr/bin/env node
let shell = require('shelljs')
let colors = require('colors')
let fs = require('fs') //fs already comes included with node.


//Extracting out the appName from the command line input
let appName = process.argv[2]
//The directory where all the files will be written
let appDirectory = `${process.cwd()}/${appName}`

// This is the main run function which will handle all the others
const run = async() => {
	let success =  await createReactApp()
	if(!success){
		console.log("Something went wrong while tring to create new React app using create-react-app".red);
		return false;		
	}
	await cdIntoNewApp()
	await installPackages()
	await updateTemplates()
	console.log("All Done ! Hurray".green);
}



//Lets make the other functions

// CreateReactApp()
// In this function we return a new promise which first checks that the user provided an 
//app name, and then executes the create-react-app command, passing in appName as 
//the first parameter. After the command completes we resolve the promise to true so our 
//script can continue.
const createReactApp = () => {
	return new Promise(resolve=>{
		if(appName){
			shell.exec(`create-react-app ${appName}`,()=>{
				console.log('create-react-app Successfully Done'.yellow);
				resolve(true)
			})
		}else{
			console.log("\nNo App name was Provided".red);
			console.log("\nGive a app name like\n create-react-redux-router-app","app-name\n".cyan);
			resolve(false)
		}
	})
}

//cdIntoNewApp()
//Once again, we return a new promise which cd's down into the directory 
//of the newly created app and then resolves to true
const cdIntoNewApp = () =>{
	return new Promise(resolve=>{
		shell.exec(`cd ${appName}`,()=>{
			console.log("CD into the new App is done".yellow);
			resolve(true);
		})
	})
} 

//Lets make the Function to install any additional packages
const installPackages =() =>{
	return new Promise(resolve=>{
		console.log("\nInstalling redux, react-router, react-router-dom, react-redux, and redux-thunk\n".cyan);
		shell.exec(`npm install redux react-router react-router-dom react-redux redux-thunk react-router-dom --save`,()=>{
			console.log("Finished Installing the additional Packages".green);
			resolve(true);
		})

	})
}


//Lets Update the templates 
//updateTemplates()
let templates = require('./templates/templates.js')

const updateTemplates = () =>{
	return  new Promise(resolve=>{
		let promises = [];
		Object.keys(templates).forEach((fileName, i)=>{
			promises[i]=new Promise(res=>{
				fs.writeFile(`${appDirectory}/src/${fileName}`,templates[fileName],function(err){
					if(err){return console.log(err)}
					res(true);
				})
			})
		})
		Promise.all(promises).then(()=>{
			console.log("All Template files Successfully Replaced".green);
			resolve(true);
		})
	})
}

run(); // Calling the run function, this must be called in index.js or the app will not run ever
